import os
import pypat
import time
from concurrent import futures

CURRENT_DIRECTORY = os.path.dirname(os.path.abspath(__file__))
VALID_AUDIO_PATH = CURRENT_DIRECTORY + '/../pypat/libpat/test/test.mp3'
MISSING_AUDIO_PATH = 'missing'
INVALID_AUDIO_PATH = CURRENT_DIRECTORY + '/../pypat/libpat/test/invalid.mp3'


def test_play_audio_file():
    pypat.play(VALID_AUDIO_PATH)


def test_play_missing_audio_file_error():
    try:
        pypat.play(MISSING_AUDIO_PATH)
        raise Exception('missing audio should not play')
    except pypat.PATException as error:
        assert 'Could not open the audio file. The audio file is inaccessible or does not exist.' == str(error)


def test_play_invalid_audio_file_error():
    try:
        pypat.play(INVALID_AUDIO_PATH)
        raise Exception('invalid audio should not play')
    except pypat.PATException as error:
        assert 'Could not demux the audio file. The audio file format is unsupported or corrupted, or '\
               'the audio file does not contain an audio stream.' == str(error)


def test_pause_resume_audio_file():
    def play_thread():
        start = time.time()

        pypat.play(VALID_AUDIO_PATH)

        end = time.time()

        elapsed = end - start

        if elapsed < 1.5:
            raise Exception('pause did not occur')

    pool = futures.ThreadPoolExecutor(max_workers=1)
    future = pool.submit(play_thread)

    # If this thread does not sleep, pause will run before the playback thread
    time.sleep(0.750)

    pypat.pause()

    time.sleep(0.750)

    pypat.resume()

    future.result()


def test_skip_audio_file():
    def play_thread():
        start = time.time()

        pypat.play(VALID_AUDIO_PATH)

        end = time.time()

        elapsed = end - start

        if elapsed > 1:
            raise Exception('skip did not occur')

    pool = futures.ThreadPoolExecutor(max_workers=1)
    future = pool.submit(play_thread)

    # If this thread does not sleep, skip will run before the playback thread
    time.sleep(0.250)

    pypat.skip()

    future.result()



import os
import setuptools.command
import setuptools.command.build_py
import shutil
import subprocess
import sys
import version

if sys.platform == 'darwin':
    _shared_lib_extension = 'dylib'
elif sys.platform == 'win32':
    _shared_lib_extension = 'dll'
else:
    _shared_lib_extension = 'so'

_libpat_build_dir = os.path.abspath(f'pypat/libpat/build')
_libpat_path = os.path.abspath(f'{_libpat_build_dir}/bin/libpat.{_shared_lib_extension}')
_libpat_file = os.path.basename(_libpat_path)


class BuildPat(setuptools.command.build_py.build_py):
    def run(self):
        subprocess.check_call(['cmake', '..', '-G', 'Ninja', '-DSKIP_PAT_PLAY_BUILD=true'], cwd=_libpat_build_dir)
        subprocess.check_call(['cmake', '--build', '.'], cwd=_libpat_build_dir)

        shutil.copy(_libpat_path, f'pypat/{_libpat_file}')

        super(BuildPat, self).run()


try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel
    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            self.root_is_pure = False
except ImportError:
    bdist_wheel = None


with open('README.md', 'r') as fh:
    long_description = fh.read()


setuptools.setup(
    name='pypat',
    version=version.VERSION,
    author='Thomas Newman',
    description = 'A Python library that makes playing audio simple.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/tnewman/pat',
    packages=setuptools.find_packages(),
    package_data={'pypat': [_libpat_file]},
    cmdclass={
        'bdist_wheel': bdist_wheel,
        'build_py': BuildPat
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Operating System :: OS Independent',
    ],
)



from ctypes import *
from enum import IntEnum, unique
import atexit
import os
import signal
import sys


def play(audio_path: str):
    """
    Play an audio file.
    :param audio_path: The path to the audio file to play. This can be a local file or remote file (http:// or https://)
    :raises PATException: Raised when PAT cannot play an audio file.
    """
    _check_error(_libpat.pat_play(_pat, c_char_p(audio_path.encode('ascii'))))


def skip():
    """
    Skip playback of the current audio file. This method does nothing if there is not an audio file playing.
    :raises PATException: Raised when PAT cannot skip playback.
    """
    _check_error(_libpat.pat_skip(_pat))


def pause():
    """
    Pause audio playback. This method does nothing if audio playback is already paused.
    :raises PATException: Raised when PAT cannot pause playback.
    """
    _check_error(_libpat.pat_pause(_pat))


def resume():
    """
    Resume audio playback. This method does nothing if audio playback is already resumed.
    :raises PATException: Raised when PAT cannot resume playback.
    """
    _check_error(_libpat.pat_resume(_pat))


class PATException(Exception):
    pass


_PAT_SUCCESS = 0


def _get_libpat_path():
    if sys.platform == 'darwin':
        _shared_lib_suffix = 'dylib'
    elif sys.platform == 'win32':
        _shared_lib_suffix = 'dll'
    else:
        _shared_lib_suffix = 'so'

    module_folder = os.path.dirname(__file__)

    libpat_paths = [f'{module_folder}/libpat/build/bin/libpat.{_shared_lib_suffix}',
                    f'{module_folder}/libpat.{_shared_lib_suffix}']
    
    for libpat_path in libpat_paths:
        if os.path.exists(libpat_path):
            return libpat_path

    raise PATException('Failed to load libpat.')


def _load_libpat():
    libpat_path = _get_libpat_path()

    libpat = cdll.LoadLibrary(libpat_path)

    libpat.pat_error_to_string.restype = c_char_p

    return libpat


def _pat_open():
    pat = c_void_p()
    _check_error(_libpat.pat_open(byref(pat)))
    return pat


def _pat_close():
    _libpat.pat_close(_pat)


def _check_error(pat_error):
    if pat_error != _PAT_SUCCESS:
        raise PATException(_libpat.pat_error_to_string(pat_error).decode('ascii'))


_libpat = _load_libpat()
_pat = _pat_open()
atexit.register(_pat_close)
